class Distributor
  module Actions

    class SetPermissions < Action
      SET_PERMISSIONS_TEMPLATE = Actions.template("set_permissions.success")

      def handle(list:, mail:, attributes:)
        return if attributes["user-email"].nil? # drop missing param
        return if attributes["permissions"].nil? # drop missing param

        modo = Email.first(mailinglist: list, email: mail.from)
        if !modo&.modo? && !modo&.op?
          $logger.warn "SECU <#{mail.from}> failed to set-permissions <#{list.email}> modo"
          return nil
        end

        user_email = attributes["user-email"]
        user = Email.first(mailinglist: list, email: user_email)
        if user.nil?
          $logger.warn "SECU <#{mail.from}> failed to set-permissions on non-existing email <#{user_email}>"
          return nil
        end

        if user.op? && !modo.op?
          $logger.warn "SECU <#{mail.from}> failed to set-permissions on op email <#{user_email}>"
          return nil
        end

        permissions = attributes["permissions"].to_i
        if Email::Permissions.op?(permissions) && !modo.op?
          $logger.warn "SECU <#{mail.from}> failed to set op permissions on email <#{user_email}>"
          return nil
        end

        user.permissions = permissions
        user.save
        body = SET_PERMISSIONS_TEMPLATE.result binding
        @distributor.distribute(
          Protocols::Mail.build(
            subject: "ML #{list.name} subscription update", list:, to: user.email, body:
          )
        )
      end
    end

    # Abstract action that check modo permissions & fetch email attr["uid"] from modo
    class ModerationDistribute < Action
      def handle_get_mail_to_distribute(list:, mail:, attributes:)
        modo = Email.first(mailinglist: list, email: mail.from)
        if !modo&.modo? && !modo&.op?
          $logger.warn "SECU <#{mail.from}> failed to validate email on <#{list.email}>"
          return nil
        end

        uid_mail = attributes["uid"]
        if !uid_mail
          $logger.warn "<#{mail.from}> did not specified email uid"
          return nil
        end

        mail_to_distribute = @distributor.imap_client.fetch_first(
          uid: uid_mail.to_i, inbox: Protocols::Imap::MODERATION_INBOX,
        )
        if !mail_to_distribute
          $logger.warn "<#{mail.from}> tried to distribute unexisting uid #{uid_mail}"
          return nil
        end

        mail_to_distribute
      end
    end

    class ValidateDistribute < ModerationDistribute
      def handle(list:, mail:, attributes:)
        mail_to_distribute = handle_get_mail_to_distribute(list:, mail:, attributes:)
        return if !mail_to_distribute

        list.enabled_readers.each do |reader|
          to_distrib = mail_to_distribute.to_redistribute(list:, dest: reader)
          @distributor.distribute(to_distrib)
        end
      end
    end

    class RefuseDistribute < ModerationDistribute
      def handle(list:, mail:, attributes:)
        mail_to_distribute = handle_get_mail_to_distribute(list:, mail:, attributes:)
        return if !mail_to_distribute

        @distributor.imap_client.move mail: mail_to_distribute, to: Protocols::Imap::REFUSED_INBOX, inbox: Protocols::Imap::MODERATION_INBOX
      end
    end

    class ListUsers < Action
    end

  end
end
